<?php

/**
 * @file
 * Page callbacks for the picasa_node_album module
 */

/**
 * Form to add albums to a node
 */
function picasa_node_album_add(&$form_state, $nid) {
  global $user;

  $form = array();
  $form['picasa'] = array(
    '#type' => 'fieldset',
    '#title' => t('Picasa Album Management')
  );

  $remove_any = user_access('remove any albums');
  $remove = $remove_any ? $remove_any : user_access('remove own albums');

  if ($remove_any) {
    $result = db_query("SELECT nid, uid, album_id, title FROM {picasa_node_album} WHERE nid = %d", $nid);
  }
  else {
    $result = db_query("SELECT nid, uid, album_id, title FROM {picasa_node_album} WHERE nid = %d AND uid = %d", $nid, $user->uid);
  }

  if ($result) {
  $form['picasa']['remove_album']['#tree'] = TRUE;
    while ($row = db_fetch_object($result)) {
      $account = user_load($row->uid);

      if ($remove) {
        $form['picasa']['remove_album'][$row->album_id] = array(
          '#type' => 'checkbox',
          '#title' => t('Remove Album: %title (posted by %user)', array('%title' => $row->title, '%user' => $account->name))
        );
      }
      else {
        $form['picasa']['remove_album'][$row->album_id] = array(
          '#value' => t('Album: %title (posted by %user)', array('%title' => $row->title, '%user' => $account->name))
        );
      }
    }
  }

  // setup user account
  $google_user = _picasa_node_album_get_user_google_account($user->uid);

  if (!$google_user) {
    $form['picasa']['google_user'] = array(
      '#value' => '<div>'. l('Setup your default Picasa account.', 'user/'. $user->uid .'/edit/picasa-node-album') .'</div>'
    );
  }

  $limit = variable_get('picasa_node_album_captions', 0);
  if ($google_user) {

    $form['picasa']['current_account'] = array(
      '#value' => t('Picasa Account: <b>%gu</b>', array('%gu' => $google_user))
    );

    $form['picasa']['picasa_album'] = array(
      '#type' => 'textfield',
      '#title' => t('Add Picasa Album'),
      '#description' => t('Start typing the name of the Picasa album here.'),
      '#size' => 30,
      '#autocomplete_path' => 'js/picasa_albums'
    );

    $form['picasa']['submit'] = array('#type' => 'submit', '#value' => t('Submit'));
  }

  $form['picasa']['nid'] = array(
    '#type' => 'hidden',
    '#value' => $nid
  );

  $form['picasa']['uid'] = array(
    '#type' => 'hidden',
    '#value' => $user->uid
  );

  $google_user_field = _picasa_node_album_get_google_user_field();
  if ($google_user_field['#type'] != 'hidden') {
    $form['account'] = array(
      '#type' => 'fieldset',
      '#title' => t('Picasa Account')
    );

    $form['account']['google_user'] = $google_user_field;
    $form['account']['switch'] = array(
    '#type' => 'submit',
    '#value' => t('Go')
    );
  }
  else {
    $form['account']['current_account'] = NULL;
  }

  $form['#validate'][] = 'picasa_node_album_validate_handler';
  $form['#submit'][] = 'picasa_node_album_submit_handler';

  return $form;
}

/**
 * submit handler for node form
 */
function picasa_node_album_validate_handler(&$form, $form_state) {
  global $user;

  if (isset($form_state['values']['google_user'])) {
    $google_user = _picasa_node_album_get_user_google_account($user->uid);

    if ($google_user == $form_state['values']['google_user']) {
      form_set_error('google_user', t('%user is already the current Picasa account.', array('%user' => $google_user)));
      return;
    }

    $exclude_accounts = variable_get('picasa_node_album_exclude_accounts', '');
    $exploded = explode("\n", $exclude_accounts);

    if (count($exploded) > 0 && $exploded[0] != '') {
      $array = array();
      // create a new array of all lower case values
      foreach ($exploded as $each) {
        $array[] = strtolower($each);
      }

      // compare lower case versions of the strings
      if (in_array(strtolower($form_state['values']['google_user']), $array)) {
        form_set_error('google_user', t('You may not use this account.'));
        return;
      }
    }
  }
}

/**
 * submit handler for node form
 */
function picasa_node_album_submit_handler(&$form, $form_state) {
  global $user;

  // load up node
  $node = node_load($form_state['values']['nid']);

  // process removals
  if (count($form_state['values']['remove_album']) > 0) {

    foreach ($form_state['values']['remove_album'] as $id => $value) {
      if ($value == 1) {
        db_query("DELETE FROM {picasa_node_album} WHERE nid = %d AND album_id = '%s'", $node->nid, $id);
        drupal_set_message(t('Album removed successfully.'));
      }
    }
  }

  if ($form_state['values']['picasa_album']) {

    // get an array of albums
    $albums = _picasa_node_album_get_albums();

    if ($album_id = array_search($form_state['values']['picasa_album'], $albums)) {
      // make sure album doesn't already exist
      $result = db_query("SELECT nid FROM {picasa_node_album} WHERE nid = %d AND album_id = %d", $node->nid, $album_id);
      if ($row = db_fetch_object($result)) {
        form_set_error('picasa_album', t('That album already exists in this node.'));
        return;
      }
      // insert record
      $google_user = _picasa_node_album_get_user_google_account($user->uid);
      $result = db_query("INSERT INTO {picasa_node_album} (nid, uid, google_user, album_id, title) VALUES (%d, %d, '%s', '%s', '%s')",
                          $node->nid, $user->uid, $google_user, $album_id, $form_state['values']['picasa_album']);
      drupal_set_message(t('Album added successfully.'));
    }
  }

  if (isset($form_state['values']['google_user'])) {
    picasa_node_album_user_edit_submit($form, $form_state);
  }
}

/**
 * Form handler for user/%/edit/picasa
 */
function picasa_node_album_user_edit($form_state, $uid) {
  // accommodate 'me' module
  if ($uid == 'me') {
    global $user;
    $uid = $user->uid;
  }

  // load account
  $account = user_load($uid);

  $form = array();
  // load google user if one exists
  $google_user = _picasa_node_album_get_user_google_account($account->uid);

  $form['text'] = array(
    '#value' => t('Setting your Google/Picasa username here will allow you to add albums to content.')
  );

  $form['uid'] = array(
    '#type' => 'hidden',
    '#value' => $account->uid
  );

  $form['google_user'] = _picasa_node_album_get_google_user_field();
  $form['google_user']['#default_value'] = $google_user;

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit')
  );

  $form['#validate'][] = 'picasa_node_album_validate_handler';

  return $form;
}

/**
 * Implementation of hook_submit().
 */
function picasa_node_album_user_edit_submit(&$form, $form_state) {
  $result = db_query("SELECT google_user FROM {picasa_node_album_users} WHERE uid = %d", $form_state['values']['uid']);
  if ($row = db_fetch_object($result)) {
    db_query("UPDATE {picasa_node_album_users} SET google_user = '%s' WHERE uid = %d", $form_state['values']['google_user'], $form_state['values']['uid']);
  }
  else {
    db_query("INSERT INTO {picasa_node_album_users} (uid, google_user) VALUES (%d, '%s')", $form_state['values']['uid'], $form_state['values']['google_user']);
  }
  drupal_set_message(t('Your Picasa settings have been updated.'));
}

/**
 * autoomplete callback for get albums
 */
function picasa_node_album_js_albums($string = '') {
  $albums = _picasa_node_album_get_albums();
  $matches = array();
  if (is_array($albums)) {
    foreach ($albums as $album) {
      if (stripos($album, $string) !== FALSE) {
        $matches[$album] = check_plain($album);
      }
    }
  }
  drupal_json($matches);
}

/**
 * Page callback for an individual album view
 */
function picasa_node_album_detail($nid, $album_id) {

  if (!$album_id) {
    $albums = _picasa_node_album_get_list($nid);
    return theme('picasa_node_album_list', $nid, $albums);
  }

  // load up album
  $result = db_query("SELECT uid, google_user, album_id, title, added, updated, images FROM {picasa_node_album} WHERE nid = %d AND album_id = '%s'", $nid, $album_id);
  $album = array();
  $cache_min = variable_get('picasa_node_album_cache_min', 60);

  if ($row = db_fetch_object($result)) {
    $account = user_load($row->uid);
    $album['postedby'] = t('posted by ') . l($account->name, 'user/'. $account->uid, array('html' => TRUE));
    $album['author'] = l($account->name, 'user/'. $account->uid, array('html' => TRUE));
    $album['title'] = $row->title;
    $album['uid'] = $account->uid;
    
    drupal_set_title($row->title);

    $cache_expires = $row->updated + $cache_min;
    if (empty($row->images) || time() > $cache_expires) {
      $album['images'] = _picasa_node_album_get_images($row->google_user, $row->album_id);
    }
    else {
      $album['images'] = unserialize($row->images);
    }

    $node_url = url('node/'. $nid);

    return theme('picasa_node_album_detail', $nid, $node_url, $album['title'], $album['uid'], $album['author'], $album['postedby'], $album['images']);
  }

  drupal_not_found();
  return;
}